@@ -4,13 +4,13 @@
 {-# LANGUAGE FlexibleContexts #-}
 {-# LANGUAGE TypeFamilies #-}
 {-# LANGUAGE CPP #-}
-#if defined(__GLASGOW_HASKELL__) && __GLASGOW_HASKELL__ >= 702
+#if __GLASGOW_HASKELL__ >= 702 && __GLASGOW_HASKELL__ <= 708
 {-# LANGUAGE Trustworthy #-}
 #endif
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Data.Profunctor.Rep
--- Copyright   :  (C) 2011-2012 Edward Kmett,
+-- Copyright   :  (C) 2011-2015 Edward Kmett,
 -- License     :  BSD-style (see the file LICENSE)
 --
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
@@ -22,10 +22,12 @@ module Data.Profunctor.Rep
   (
   -- * Representable Profunctors
     Representable(..), tabulated
+  , firstRep, secondRep
   -- * Corepresentable Profunctors
   , Corepresentable(..), cotabulated
   ) where
 
+import Control.Applicative
 import Control.Arrow
 import Control.Comonad
 import Data.Functor.Identity
@@ -37,11 +39,19 @@ import Data.Tagged
 
 -- | A 'Profunctor' @p@ is 'Representable' if there exists a 'Functor' @f@ such that
 -- @p d c@ is isomorphic to @d -> f c@.
-class (Functor (Rep p), Profunctor p) => Representable p where
+class (Functor (Rep p), Strong p) => Representable p where
   type Rep p :: * -> *
   tabulate :: (d -> Rep p c) -> p d c
   rep :: p d c -> d -> Rep p c
 
+-- | Default definition for 'first'' given that p is 'Representable'.
+firstRep :: Representable p => p a b -> p (a, c) (b, c)
+firstRep p = tabulate $ \(a,c) -> (\b -> (b, c)) <$> rep p a
+
+-- | Default definition for 'second'' given that p is 'Representable'.
+secondRep :: Representable p => p a b -> p (c, a) (c, b)
+secondRep p = tabulate $ \(c,a) -> (,) c <$> rep p a
+
 instance Representable (->) where
   type Rep (->) = Identity
   tabulate f = runIdentity . f
@@ -62,6 +72,13 @@ instance Functor f => Representable (UpStar f) where
   {-# INLINE tabulate #-}
   rep = runUpStar
   {-# INLINE rep #-}
+  
+instance Representable (Forget r) where
+  type Rep (Forget r) = Const r
+  tabulate = Forget . (getConst .)
+  {-# INLINE tabulate #-}
+  rep = (Const .) . runForget
+  {-# INLINE rep #-}
 
 type Iso s t a b = forall p f. (Profunctor p, Functor f) => p a (f b) -> p s (f t)
 
@@ -115,7 +132,7 @@ instance Functor f => Corepresentable (DownStar f) where
 --
 -- This can be used with the combinators from the @lens@ package.
 --
--- @'tabulated' :: 'Corep' f p => 'Iso'' (f d -> c) (p d c)@
+-- @'cotabulated' :: 'Corep' f p => 'Iso'' (f d -> c) (p d c)@
 cotabulated :: (Corepresentable p, Corepresentable q) => Iso (Corep p d -> c) (Corep q d' -> c') (p d c) (q d' c')
 cotabulated = dimap cotabulate (fmap corep)
 {-# INLINE cotabulated #-}
