@@ -1,7 +1,10 @@
 {-# LANGUAGE CPP #-}
-#if defined(__GLASGOW_HASKELL__) && __GLASGOW_HASKELL__ >= 702
+#if __GLASGOW_HASKELL__ >= 708
+{-# LANGUAGE Trustworthy #-}
+#elif __GLASGOW_HASKELL__ >= 702
 {-# LANGUAGE Unsafe #-}
 #endif
+{-# LANGUAGE ScopedTypeVariables #-}
 -----------------------------------------------------------------------------
 -- |
 -- Copyright   :  (C) 2011-2013 Edward Kmett
@@ -37,7 +40,12 @@ import Control.Comonad (Cokleisli(..))
 import Control.Monad (liftM)
 import Data.Tagged
 import Prelude hiding (id,(.),sequence)
+
+#if __GLASGOW_HASKELL__ >= 708
+import Data.Coerce
+#else
 import Unsafe.Coerce
+#endif
 
 {-# ANN module "Hlint: ignore Redundant lambda" #-}
 {-# ANN module "Hlint: ignore Collapse lambdas" #-}
@@ -127,7 +135,11 @@ class Profunctor p where
   -- should match the default definition:
   --
   -- @('Profuctor.Unsafe.#.') ≡ \\f -> \\p -> p \`seq\` 'rmap' f p@
+#if __GLASGOW_HASKELL__ >= 708
+  ( #. ) :: Coercible c b => (b -> c) -> p a b -> p a c
+#else
   ( #. ) :: (b -> c) -> p a b -> p a c
+#endif
   ( #. ) = \f -> \p -> p `seq` rmap f p
   {-# INLINE ( #. ) #-}
 
@@ -153,10 +165,18 @@ class Profunctor p where
   -- operationally identity.
   --
   -- @('.#') ≡ \\p -> p \`seq\` \\f -> 'lmap' f p@
+#if __GLASGOW_HASKELL__ >= 708
+  ( .# ) :: Coercible b a => p b c -> (a -> b) -> p a c
+#else
   ( .# ) :: p b c -> (a -> b) -> p a c
+#endif
   ( .# ) = \p -> p `seq` \f -> lmap f p
   {-# INLINE ( .# ) #-}
 
+#if __GLASGOW_HASKELL__ >= 708
+  {-# MINIMAL dimap | (lmap, rmap) #-}
+#endif
+
 instance Profunctor (->) where
   dimap ab cd bc = cd . bc . ab
   {-# INLINE dimap #-}
@@ -164,9 +184,14 @@ instance Profunctor (->) where
   {-# INLINE lmap #-}
   rmap = (.)
   {-# INLINE rmap #-}
+#if __GLASGOW_HASKELL__ >= 708
+  ( #. ) _ = coerce (\x -> x :: b) :: forall a b. Coercible b a => a -> b
+  ( .# ) pbc _ = coerce pbc
+#else
   ( #. ) _ = unsafeCoerce
-  {-# INLINE ( #. ) #-}
   ( .# ) pbc _ = unsafeCoerce pbc
+#endif
+  {-# INLINE ( #. ) #-}
   {-# INLINE ( .# ) #-}
 
 instance Profunctor Tagged where
@@ -176,7 +201,11 @@ instance Profunctor Tagged where
   {-# INLINE lmap #-}
   rmap = fmap
   {-# INLINE rmap #-}
+#if __GLASGOW_HASKELL__ >= 708
+  ( #. ) _ = coerce (\x -> x :: b) :: forall a b. Coercible b a => a -> b
+#else
   ( #. ) _ = unsafeCoerce
+#endif
   {-# INLINE ( #. ) #-}
   Tagged s .# _ = Tagged s
   {-# INLINE ( .# ) #-}
@@ -189,7 +218,11 @@ instance Monad m => Profunctor (Kleisli m) where
   rmap k (Kleisli f) = Kleisli (liftM k . f)
   {-# INLINE rmap #-}
   -- We cannot safely overload (#.) because we didn't provide the 'Monad'.
+#if __GLASGOW_HASKELL__ >= 708
+  ( .# ) pbc _ = coerce pbc
+#else
   ( .# ) pbc _ = unsafeCoerce pbc
+#endif
   {-# INLINE ( .# ) #-}
 
 instance Functor w => Profunctor (Cokleisli w) where
@@ -200,5 +233,9 @@ instance Functor w => Profunctor (Cokleisli w) where
   rmap k (Cokleisli f) = Cokleisli (k . f)
   {-# INLINE rmap #-}
   -- We cannot safely overload (.#) because we didn't provide the 'Functor'.
+#if __GLASGOW_HASKELL__ >= 708
+  ( #. ) _ = coerce (\x -> x :: b) :: forall a b. Coercible b a => a -> b
+#else
   ( #. ) _ = unsafeCoerce
+#endif
   {-# INLINE ( #. ) #-}
